/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/JobStatusPod", [
    "ibm/tivoli/tip/TCTTable",
    "ibm/tivoli/tip/TCTNavModel",
    "dojo/_base/kernel",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dijit/PopupMenuBarItem",
    "ibm/usmi/util/DefaultMessageHandler",
    "./JobStatusDataSet",
    "./StatusPodMenuDialog",
    "./JobStatusPodDialogContent",
    "dojo/DeferredList",
    "dojo/_base/Deferred",
    'dojo/_base/unload',
    "./Constants",
    "dojo/_base/connect",
    "dojo/string",
    "dojo/i18n!./nls/statuspodResources",
    "ibm/usmi/core/utils/Logger"
], function(TCTTable, TCTNavModel, dojo, declare, lang, PopupMenuBarItem, DefaultMessageHandler, Dataset,
        StatusPodMenuDialog, JobStatusPodDialogContent, DeferredList, Deferred, baseUnload, Constants, connect, string,
        MRI, Logger) {
    return declare("ibm.usmi.statuspod.JobStatusPod", [ PopupMenuBarItem
    ], {
        // summary:
        //      Job Status Pod widget 
        // description:
        //      The Job status pod widget would show the jobs with various status like Error/Active/Completed/Scheduled.
        //      It would show the jobs in the same look and feel as mega menu.

        label: MRI.JobMenuBarItemLoadingLabel,

        dialog: null,

        dataset: null,

        jobsCallback: null,

        jobDatasetName: "SCHEDULER_STATUSPOD",

        sortProp: [ {
            attribute: "LAST_RUN",
            descending: true
        }
        ],

        sortPropGetItems: "-LAST_RUN",

        totalCount: 0,

        iconClass: "status-msgs-icon-confirmation",

        _logger: new Logger("ibm.usmi.statuspod", "JobStatusPod"),

        blockEvents: true,
        pendingEvents: 0,

        postCreate: function() {
            this.inherited(arguments);
            this.dialog = new JobStatusPodDialogContent();
            this.popup = new StatusPodMenuDialog({
                content: this.dialog
            });
        },

        startup: function() {
            this._logger.finest("startup job status pod", "startup");
            this.inherited(arguments);
            connect.subscribe(Constants.updateJobsLabel, this, function(result) {
                this.updateLabel(result.errorJobs, result.totalJobs);
            });
            this.jobsCallback = lang.hitch(this.dialog, this.dialog.updateResult);
            this.makeDataSet();
            connect.subscribe(Constants.TOPIC_CLOSE_JOBS_DIALOG, this, function(result) {
                this.popup.close();
            });
            
            //#d198051: need to create the timer to unblock event handling here
            connect.subscribe(Constants.TOPIC_UNBLOCKEVENTS, this, function(result) {
                this.unblockEvents();
            });
        },

        refreshCallback: function(listenerObj, event) {
            // summary:
            //     refresh the job status pod based on the event
            // listenerObj: Object
            //      unique object
            // event: ibm.tivoli.navmodel.Event
            //      the event received
            var log = this._logger, deferred;
            log.finest("job status pod refresh callback was called", "refreshCallback");
            //if job progress update, directly setting job progress w/o re-fetching the items
            //but if the job has pending event, we need to re-fetching the items at this time
            if (event && !this._hasPendingEvent()) {
                switch (event.getType()) {
                case "itemPropertiesChanged":
                    var changedProps = event.getProperties();
                    var items = event.getItems();
                    var idxProps, idxItems, item, props, prop;
                    for (idxItems = 0; idxItems < items.length; idxItems++) {
                        item = items[idxItems];
                        props = changedProps[idxItems];
                        for (idxProps = 0; idxProps < props.length; idxProps++) {
                            prop = props[idxProps];
                            if (prop.getId() === "PROGRESS") {
                                this.dialog.updateItemProperties(item.getId(), props);
                                log.finest("job status pod updated properties", "refreshCallback");
                                return;
                            }
                        }
                    }
                    break;
                default:
                    break;
                }
            }
            if (this.blockEvents) { //Refresh is in progress, don't refresh
                this._logger
                        .finest("job status pod refresh callback is blocking, set pending state", "refreshCallback");
                this.pendingEvents = this.pendingEvents + 1; //save the need to refresh again later
                return;
            }
            this.blockEvents = true;
            this.pendingEvents = 0;
            this.label = MRI.JobPodRefreshing;
            this.containerNode.innerHTML = this.label;
            this.dialog.clearDescendants();
            this.fetchItems(this.jobsCallback);
            log.finest("job status pod re-fecthed the items", "refreshCallback");

        },

        _hasPendingEvent: function() {
            // summary:
            //     check if there are pending events need to be handled
            // returns:
            //     true if there are pending events, otherwise false.   
            var hasPendingEvent = false;
            if (this.pendingEvents > 0) {
                hasPendingEvent = true;
            }
            return hasPendingEvent;
        },

        _unblockEvents: function() {
            // summary:
            //     unblock the event handling
            if(this.dialog.forceRefresh===true){
                    //force refresh
                    this.dialog.forceRefresh=false;
                    this._forceRefresh();
                    return;
            }
            this._logger.finest("job status pod unblock the event refreshing", "_unblockEvents");
            this.blockEvents = false; //Refresh is done, we can now listen for events again
            if (this._hasPendingEvent()) { //did any more events come in during our refresh? if so, refresh again
                this._logger.finest("there are pending events, begine to refresh", "_unblockEvents");
                this.refreshCallback();
            }
        },
        
        _forceRefresh: function() {
            // summary:
            //    recreate the dataset and populated the menu items
            if(this.dataset){
                this.dataset.destroy();
            }
            if(this.dialog) {
                this.dialog.clearDescendants();
            }
            this.makeDataSet();
        },
        
        unblockEvents: function() {
            // summary:
            //     create timer to unblocking events with the delay
            var delay = Constants.RERESH_DELAY;
            if (this.pendingEvents >= Constants.LONG_RERESH_THRESHOLD) {
                //if too many pending events, then use large delay
                delay = Constants.LONG_RERESH_DELAY;
            }
            this._logger.finest("creating unblockEvents timer", "unblockEvents");
            window.setTimeout(lang.hitch(this, this._unblockEvents), delay);
        },

        makeDataSet: function() {
            // summary:
            //      create the dataset for job status pod
            // returns:
            //      dataset for the job status pod

            var i, dataset;
            dataset = new Dataset({
                baseUrl: "/ibm/tivoli/rest",
                provider: "USMI",
                datasource: "",
                dataset: this.jobDatasetName,
                clearCacheOnRefresh: true,
                onComplete: lang.hitch(this, function(dataset, kwArgs) {
                    this._logger.finest("dataset loading completed ", "makeDataSet");
                    var callback = this.jobsCallback, handler;
                    this.dataset = dataset;
                    this.fetchItems(callback);
                }),
                onError: function(error, exception, ioArgs) {
                    var msgHandler = new DefaultMessageHandler({});
                    msgHandler.displayException(exception);
                },
                parameters: {
                    maxSPElementsToken: 10
                },
                defaultProperties: []
            });
            this.dataset = dataset;
            this.dialog.dataset = dataset;
            dataset.addEventListener(this, lang.hitch(this, this.refreshCallback));

        },


        destroy: function() {
            // summary:
            //      destroy the job status pod and it's corresponding dataset
            if (this.dataset) {
                this.dataset.destroy();
                this.dataset = null;
            }
            if (this.popup) {
                this.popup.close();
            }
            if (this.dialog) {
                this.dialog.destroyRecursive();
            }
            this.inherited(arguments);
        },

        fetchItems: function(callback) {
            // summary:
            //      fetching the items from job status pod dataset
            // returns: 
            //      the items list of the dataset
            // callback: Function
            //      callback method for the items fetching
            this.dataset.getItems({
                onComplete: lang.hitch(this, function(items, kwArgs) {
                    callback(items);
                    //sent unblock event handling notification
                    connect.publish(Constants.TOPIC_UNBLOCKEVENTS);
                }),
                onError: function(error, exception, ioArgs) {
                    var msgHandler = new DefaultMessageHandler({});
                    msgHandler.displayException(exception);
                    //sent unblock event handling notification
                    connect.publish(Constants.TOPIC_UNBLOCKEVENTS);
                },
                start: 0,
                scope: this,
                sort: this.sortPropGetItems,
                properties: [
                    "NAME",
                    "LAST_RUN_STATUS",
                    "LAST_RUN",
                    "STATUS",
                    "TASK",
                    "PROGRESS",
                    "CREATED_BY",
                    "NEXT_RUN",
                    "DESCRIPTION",
                    "categoryId",
                    "categoryCount"
                ]
            });
        },

        updateLabel: function(errorJobs, totalJobs) {
            // summary:
            //      update the menu bar icon and label
            // errorJobs: Integer
            //      the number of the error jobs
            // totalJobs: Integer
            //      the number of the total jobs
            this.totalCount = totalJobs;
            if (errorJobs && errorJobs > 0) {
                this.iconClass = "status-msgs-icon-error";
            }
            if (errorJobs === 0) {
                this.iconClass = "status-msgs-icon-confirmation";
            }
            this.label = string.substitute("<span class='${iconClass}'></span>", {
                iconClass: this.iconClass
            });
            this.label = this.label + lang.replace(MRI.JobMenuBarItemLabel, [ this.totalCount
            ]);
            this.containerNode.innerHTML = this.label;
        }

    });
});
