/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define(
        "ibm/usmi/util/DefaultURLLaunchTask", [
            "dojo/_base/declare",
            "dojo/_base/lang",
            "dojo/topic",
            "ibm/usmi/util/DefaultMessageHandler",
            "ibm/usmi/core/utils/Logger",
            "ibm/tivoli/navmodel/Selection"
        ],
        function(declare, lang, topic, DefaultMessageHandler, Logger, Selection) {

            return declare(
                    [],
                    {

                        // private members
                        _launchParms: null,
                        _menuItem: null,
                        _context: null,

                        // the logger
                        _defaultURLLaunchTaskLogger: new Logger("ibm.usmi.util", "DefaultURLLaunchTask"),

                        constructor: function() {

                            // Register for a URL launch task event
                            topic.subscribe("DefaultURLLaunch", lang.hitch(this, this._handleURLLaunch));

                        },

                        _handleURLLaunch: function(launchParms, menuItem, context) {
                            // summary:
                            //     This method is called when the "DefaultURLLaunch topic is published as a 
                            //     result of an action selected that launches a URL.
                            // description:
                            //     Calls the launchURL method passing the launchParms.  Also handles special cases
                            //     where the task must be run before the call to launchURL.

                            this._launchParms = launchParms;
                            this._menuItem = menuItem;
                            this._context = context;

                            // This method fetches the group export task, runs the task, and then launches the url.
                            var runServerTaskAndLaunchURL = function(tasks) {
                                var task = null;

                                var launchUrl = function() {
                                    this.launchURL(this._launchParms);
                                };

                                // Get the selected task from all available tasks
                                task = this.fetchTask(tasks);

                                // Call execute on the server task
                                task.execute({
                                    onComplete: lang.hitch(this, launchUrl),
                                    onError: lang.hitch(this, this._handleExecuteOnServerError),
                                    selection: this.getSelection(this._context)
                                });
                            };

                            // Check for special cases where we need to call execute on task before launching the url.
                            if (menuItem && menuItem.actionID === "GroupExport") {

                            	// set the task
                                context.dataset.getTasks({
                                    onComplete: lang.hitch(this, runServerTaskAndLaunchURL),
                                    onError: lang.hitch(this, this.getTasksError),
                                    taskCategory: "menu",
                                    selection: this.getSelection(context)
                                });
                            } else {
                                this.launchURL(launchParms);
                            }
                        },

                        // Gets the task based on the actionID passed in
                        fetchTask: function(tasks) {

                            //Want to loop through tasks looking for match to menuItem actionID
                            if (tasks) {
                                for ( var i = 0; i < tasks.length; i++) {
                                    if (tasks[i].getId() === this._menuItem.actionID) {
                                        return tasks[i];
                                    } else if (tasks[i].getChildTasks()) {
                                        var subTasks = tasks[i].getChildTasks();
                                        if (subTasks.length > 0) {
                                            var task = this.fetchTask(subTasks);
                                            if (task && (task.getId() === this._menuItem.actionID)) {
                                                return task;
                                            }
                                        }
                                    }
                                }
                            }
                        },

                        // This method does the appropriate launch based on the launch type pased in as part of the 
                        // launchParms.
                        launchURL: function(launchParms) {

                            // Check launchParms to decide how to handle
                            if (launchParms) {
                                // ISCPage launch
                                if (launchParms.launchType === "ISCPage") {
                                    try {
                                        if (this._context.selection && !(this._context.selection.isEmpty())) {
                                            var targets = this._context.selection.getItemIds();
											if (!launchParms.urlParms) {
												launchInContext(launchParms.pageID, targets, false);
											} else {
												launchInContext(launchParms.pageID, targets, false, launchParms.urlParms);
											}						
                                        } else {
                                            launchInContext(launchParms.pageID);
                                        }
                                    } catch (ex) {
                                        //There was a problem launching the window
                                        var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                                        msgHandler.displayException(ex);
                                    }
                                }

                                // URL launch
                                else if (launchParms.launchType === "URL" || launchParms.launchType === "URLPost") {
                                    if (launchParms.launchErrorMessage){
                                        //launch error, show the error message
                                        var msgHandler, msgId = "", msgSeverity = "error", theMessageTitle = launchParms.launchErrorMessage; 
                                        msgHandler = new DefaultMessageHandler();
                                        msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle, null, true);
                                        
                                    } else if (launchParms.urlToLaunchParameters) {
                                        try {
                                            var WindowObjectReference = submitRemoteAction( 
                                                    launchParms.urlToLaunch, launchParms.urlToLaunchParameters, launchParms.windowId);
                                            WindowObjectReference.focus();
                                                
                                        } catch (ex) {
                                            //There was a problem launching the window
                                            var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                                            msgHandler.displayException(ex);
                                        }
                                        
                                    } else if (launchParms.urlToLaunch) {
                                        if (launchParms.windowId) {
                                            try {
                                                var openWindow = window.open(launchParms.urlToLaunch,
                                                        launchParms.windowId);
                                                openWindow.focus();
                                            } catch (ex) {
                                                //There was a problem launching the window
                                                var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                                                msgHandler.displayException(ex);
                                            }
                                        } else {
                                            try {
                                                var openWindow = window.open(launchParms.urlToLaunch);
                                                openWindow.focus();
                                            } catch (ex) {
                                                //There was a problem launching the window
                                                var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                                                msgHandler.displayException(ex);
                                            }
                                        }
                                    } else {
                                        if (this._defaultURLLaunchTaskLogger.isWarn()) {
                                            this._defaultURLLaunchTaskLogger.warn(
                                                    "launchParms are not valid - urlToLaunch not defined", "launchURL");
                                        }
                                    }
                                }

                                // Widget launch, used by DefaultDojoUrlTask, were we are loading new widgets
                                else if (launchParms.launchType === "Widget") {

                                    if (launchParms.urlParameters) {
                                        // Launch the widget within the framework
                                        try {
                                            launchNextGenPage(launchParms.urlParameters);
                                        } catch (ex) {
                                            //There was a problem launching the window
                                            var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                                            msgHandler.displayException(ex);
                                        }
                                    } else {
                                        if (this._defaultURLLaunchTaskLogger.isWarn()) {
                                            this._defaultURLLaunchTaskLogger.warn(
                                                    "launchParms are not valid - urlParameters not defined",
                                                    "launchURL");
                                        }
                                    }
                                } else {
                                    if (this._defaultURLLaunchTaskLogger.isWarn()) {
                                        this._defaultURLLaunchTaskLogger.warn(
                                                "launchParms are not valid - launchType not define or not valid",
                                                "launchURL");
                                    }
                                }

                            } else {
                                if (this._defaultURLLaunchTaskLogger.isWarn()) {
                                    this._defaultURLLaunchTaskLogger.warn("launchParms are not defined", "launchURL");
                                }
                            }
                        },

                        // Creates and returns a new Selection object based on selection passed in on context
                        getSelection: function(context) {
                            var selectedThings = context.selection;
                            var currentSelection = null;
                            if (selectedThings) {
                                currentSelection = new Selection({
                                    version: context.dataset.getVersion()
                                });
                                currentSelection.addItems(selectedThings.getItemIds());
                                currentSelection.addLinks(selectedThings.getLinkIds());
                                currentSelection.addRelationships(selectedThings.getRelationshipIds());
                            }
                            return currentSelection;
                        },

                        // Displays any error returned if getTasks fails
                        getTasksError: function(error, request) {

                            if (this._defaultURLLaunchTaskLogger.isError()) {
                                this._defaultURLLaunchTaskLogger
                                        .error(">>> error getting tasks from server = " + error,
                                                "_handleExecuteOnServerError");
                            }

                            //There was a problem getting the tasks from the dataset
                            var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                            msgHandler.displayException(error);
                        },

                        // Displays an error returned if task execute fails
                        _handleExecuteOnServerError: function(error, exception, ioArgs) {

                            if (this._defaultURLLaunchTaskLogger.isError()) {
                                this._defaultURLLaunchTaskLogger.error(">>> error executing server task = " + error,
                                        "_handleExecuteOnServerError");
                            }

                            //There was a problem executing the task on the server
                            var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                            msgHandler.displayException(exception);
                        }
                    });
        });