/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

/*
widget that keeps the current session alive by calling the ISC login 
page to show that the session is still active. A call to an ISC page
is neccessary because NGUI uses the session created by ISC. How often the 
widget makes a call to the ISC is set by the variable SESSION_KEEP_ALIVE_INTERVAL
and the page that is called set by SESSION_KEEP_ALIVE_PAGE
NOTE: if NGUI moves away from getting the session from ISC this will have to be 
updated to request a page behind a NGUI secured URL
This widget also detects when the current session ends and publishes a logout alert
so that any open NGUI/ISC-based pages redirect to the login screen
*/
define("ibm/usmi/widgets/util/SessionKeepAlive", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/topic",
	"dijit/_WidgetBase",
	"ibm/usmi/core/utils/Logger"
], function (declare, lang, topic, _WidgetBase, Logger) {
	return declare([ _WidgetBase
	],
			{
				//set the keep alive interval to 10 minutes
				_SESSION_KEEP_ALIVE_INTERVAL : 600000,
				_SESSION_KEEP_ALIVE_PAGE : "/ibm/console/index.do",
				//stores the interval ID, returned by the setInterval call
				_intervalID : null,
				_sessionKeepAliveLogger : new Logger("ibm.usmi.widgets.util", "SessionKeepAlive"),

				postCreate : function () {
					this.inherited(arguments);
					this._intervalID = setInterval(dojo.hitch(this, this._sessionKeepAlive),
							this._SESSION_KEEP_ALIVE_INTERVAL);

				},

				_sessionKeepAlive : function () {
					if (this._sessionKeepAliveLogger.isFinest()) {
						this._sessionKeepAliveLogger.entering("", "_sessionKeepAlive");
					}
					var xhrArgs, deferred;
					xhrArgs = {
						url : this._SESSION_KEEP_ALIVE_PAGE,
						load : dojo.hitch(this, this._sessionKeepAliveRequestLoad),
						error : dojo.hitch(this, this._sessionKeepAliveRequestError)
					};

					deferred = dojo.xhrGet(xhrArgs);

					if (this._sessionKeepAliveLogger.isFinest()) {
						this._sessionKeepAliveLogger.exiting("", "_sessionKeepAlive");
					}
				},

				/*
					handles the session keep alive response. Checks to see if the data returned
					contains the text "logonForm" which means that ISC returned the login page
					meaning that the session ended on the ISC side. If that's the case, a logout
					event is published
				*/
				_sessionKeepAliveRequestLoad : function (data) {
					if (this._sessionKeepAliveLogger.isFinest()) {
						this._sessionKeepAliveLogger.entering("", "_sessionKeepAliveRequestLoad");
					}
					if (data) {
						//the ISC login form was returned. The session has ended so publish a logout event
						if (data.indexOf("logonForm") > 0) {
							topic.publish("logout-event", null);
						}
					}
					if (this._sessionKeepAliveLogger.isFinest()) {
						this._sessionKeepAliveLogger.exiting("", "_sessionKeepAliveRequestLoad");
					}
				},

				/*
					handles the session keep alive response when there was an 
					error when making the request. If an error occured, this means
					there was an issue communicating with the server so 
					publish a logout event
				*/
				_sessionKeepAliveRequestError : function () {
					if (this._sessionKeepAliveLogger.isFinest()) {
						this._sessionKeepAliveLogger.entering("", "_sessionKeepAliveRequestError");
					}
					topic.publish("logout-event", null);
					if (this._sessionKeepAliveLogger.isFinest()) {
						this._sessionKeepAliveLogger.exiting("", "_sessionKeepAliveRequestError");
					}
				},

				destroy : function () {
					if (this._intervalID) {
						clearInterval(this._intervalID);
					}

					this.inherited(arguments);
				},

				destroyRecursive : function () {
					this.inherited(arguments);
				}
			});
});
